local Lib = MissionMinderPreLoad;
local DEBUG = Lib.debug;
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

Lib.hooks = {};
local module = Lib.hooks;


local GARRISON_CACHE = 10
local function garrisonCacheLooted(event, ...)
	local typeIdentifier, itemLink, quantity, specID, sex, isPersonal, lootSource = ...;
	if ( lootSource == GARRISON_CACHE ) then
		Lib.callbacks:Fire("GARRISON_CACHE_LOOTED", quantity)
	end
end

local function shipmentLooted(event)
	if ( IsMapGarrisonMap(GetCurrentMapAreaID()) ) then -- Only fire while in garrison
		C_Garrison.RequestLandingPageShipmentInfo();
	end
end

local function shipmentCrafterInfo(event, ...)
	local success, _, maxShipments, plotID = ...

	if ( success ) then
		Lib.callbacks:Fire("WORKORDER_UPDATE_AVAILABLE", plotID, maxShipments)
	end
end

local function workorderFrameClosed(event, ...)
	C_Timer.After(1, C_Garrison.RequestLandingPageShipmentInfo)
end

local function shipmentUpdateAvailable(event, ...)
	Lib.callbacks:Fire("SHIPMENT_UPDATE_AVAILABLE")
end

--[===[@alpha@
local function debugEvent(...)
	print("[EVENT]", ...)
end
--@end-alpha@]===]

local function startBuilding(event, plotID)
	Lib.callbacks:Fire("BUILDING_UPDATE_AVAILABLE", plotID)
end

local function removedBuilding(event, plotID, buildingID)
	Lib.callbacks:Fire("BUILDING_UPDATE_AVAILABLE", plotID)
	C_Garrison.RequestLandingPageShipmentInfo(); -- need to update workorders for the removed building
end

local function activateBuilding(event, plotID, buildingID)
	Lib.callbacks:Fire("BUILDING_UPDATE_AVAILABLE", plotID)
	C_Garrison.RequestLandingPageShipmentInfo(); -- need to update workorders for the activated building
end

local rushOrderSpells = {
	[180704] = true, -- Visual
	[181507] = true, [181229] = true, [181228] = true, -- Alchemy Lab
	[180905] = true, -- Barn
	[181215] = true, -- Dwarven Bunker
	[181524] = true, [181233] = true, [181232] = true, --Enchanter's Study
	[181234] = true, [181235] = true, [181525] = true, -- Engineering Works
	[181220] = true, [181219] = true, -- Garden Shipment
	[181238] = true, [181522] = true, [181239] = true, -- Gem Boutique
	[181244] = true, -- Gladiator's Sanctum
	[181249] = true, -- Gnomish Gearworks
	[181250] = true, -- Goblin Workshop
	[180906] = true, -- Lumber Mill
	[181221] = true, [181222] = true, -- Mine Shipment
	[181519] = true, [181236] = true, [181237] = true, -- Scribe's Quarters
	[181242] = true, [181243] = true, [181521] = true, -- Tailoring Emporium
	[181230] = true, [181520] = true, [181231] = true, -- The Forge
	[181518] = true, [181241] = true, [181240] = true, -- The Tannery
	[181246] = true, [181245] = true, -- Trading Post
	[181217] = true -- War Mill
}

local function checkForRushOrder(event, unit, name, rank, castID, spellID)
	if ( rushOrderSpells[spellID] ) then
		Lib.callbacks:Fire("RUSH_ORDER_USED", spellID)
	end
end

local function initialize()
	DEBUG("   |cffffffffpreload-Hooks|r initialized");

	-- GARRISON_CACHE_LOOTED
	Lib:RegisterEvent("hooks", "SHOW_LOOT_TOAST", garrisonCacheLooted)

	-- WORKORDER_UPDATE_AVAILABLE
	Lib:RegisterEvent("hooks", "SHIPMENT_CRAFTER_INFO", shipmentCrafterInfo)

	-- SHIPMENT_UPDATE_AVAILABLE
	Lib:RegisterEvent("hooks", "LOOT_CLOSED", shipmentLooted)
	Lib:RegisterEvent("hooks", "SHIPMENT_CRAFTER_CLOSED", workorderFrameClosed)
	Lib:RegisterEvent("hooks", "GARRISON_LANDINGPAGE_SHIPMENTS", shipmentUpdateAvailable)

	-- BUILDING_UPDATE_AVAILABLE
	Lib:RegisterEvent("hooks", "GARRISON_BUILDING_PLACED", startBuilding)
	Lib:RegisterEvent("hooks", "GARRISON_BUILDING_REMOVED", removedBuilding)
	Lib:RegisterEvent("hooks", "GARRISON_BUILDING_ACTIVATED", activateBuilding)

	-- RUSH_ORDER_USED
	Lib:RegisterEvent("hooks", "UNIT_SPELLCAST_SUCCEEDED", checkForRushOrder)

	--[===[@alpha@
	Lib.RegisterCallback("_hooks", "GARRISON_CACHE_LOOTED", debugEvent)
	Lib.RegisterCallback("_hooks", "SHIPMENT_UPDATE_AVAILABLE", debugEvent)
	Lib.RegisterCallback("_hooks", "WORKORDER_UPDATE_AVAILABLE", debugEvent)
	Lib.RegisterCallback("_hooks", "BUILDING_UPDATE_AVAILABLE", debugEvent)
	Lib.RegisterCallback("_hooks", "RUSH_ORDER_USED", debugEvent)
	--@end-alpha@]===]
end

Lib:RegisterEvent("hooks", "ADDON_LOADED", initialize);
